import shutil
import subprocess
import tempfile
import zipfile
from pathlib import Path
from typing import Any
from urllib.request import urlopen

try:
    import tomllib  # pyright: ignore[reportMissingTypeStubs]
except ImportError:  # pragma: no cover
    import toml as tomllib

from hatch_openzim.shared import logger

DEFAULT_OPENZIM_TOML_LOCATION = "openzim.toml"


def process(openzim_toml_location: str = DEFAULT_OPENZIM_TOML_LOCATION):
    """performs openZIM operations on files, i.e. download and extract ZIPs

    Configuration is read from openzim.toml which must be in root of the hatch project,
    next to pyproject.toml

    openzim_toml_location: location of the openzim.toml file with instructions about
    files to install locally
    """
    config_path = Path(openzim_toml_location)
    if not config_path.exists():
        if openzim_toml_location != DEFAULT_OPENZIM_TOML_LOCATION:
            raise Exception(f"File is missing at {openzim_toml_location}")
        else:
            return
    config = tomllib.loads(config_path.read_text())
    files_config = config.get("files", None)
    if not files_config:
        return

    for section_name, section_data in files_config.items():
        _process_section(section_name=section_name, section_data=section_data)


def _process_section(section_name: str, section_data: dict[str, Any]):
    """processes all actions required for one section (i.e. one target folder)"""

    logger.info(f"Processing {section_name} section")
    section_config = section_data.get("config", None)
    if not section_config:
        raise Exception("config table is mandatory")
    base_target_dir = section_config.get("target_dir", None)
    if not base_target_dir:
        raise Exception("target_dir is mandatory in config table")
    base_target_dir = Path(base_target_dir)
    section_actions = section_data.get("actions", None)
    if not section_actions:
        logger.info("  No actions to process")
        return
    base_target_dir = Path(section_config["target_dir"])
    base_target_dir.mkdir(parents=True, exist_ok=True)
    logger.info(f"  Installing files in {base_target_dir}")
    for action_name, action_config in section_actions.items():
        _process_one_action(base_target_dir, action_name, action_config)

    execute_after = section_config.get("execute_after", None)
    if execute_after:
        _process_execute_after(base_target_dir=base_target_dir, actions=execute_after)

    logger.info(" All done")


def _process_one_action(
    base_target_dir: Path, action_name: str, action_data: dict[str, Any]
):
    """processes one action (basically trigger the right kind of action)"""
    logger.info(f"  Processing {action_name} action")
    source = action_data.get("source", None)
    if not source:
        raise Exception("source is not configured")
    action = action_data.get("action", None)
    if not action:
        raise Exception("action is not configured")

    if action == "get_file":
        _process_get_file_action(
            base_target_dir=base_target_dir, source=source, action_data=action_data
        )

    elif action == "extract_all":
        _process_extract_all_action(
            base_target_dir=base_target_dir, source=source, action_data=action_data
        )

    elif action == "extract_items":
        _process_extract_items_action(
            base_target_dir=base_target_dir, source=source, action_data=action_data
        )

    else:
        raise Exception(f"Unsupported action '{action}'")

    execute_after = action_data.get("execute_after", None)
    if execute_after:
        _process_execute_after(base_target_dir=base_target_dir, actions=execute_after)

    logger.info("    Done")


def _process_execute_after(base_target_dir: Path, actions: list[str]):
    """execute actions after file(s) installation"""

    for action in actions:
        logger.info(f"  Executing '{action}'")
        process = subprocess.run(  # noqa: PLW1510,S602
            action,
            shell=True,  # nosec: B602
            cwd=base_target_dir,
            text=True,
            stderr=subprocess.STDOUT,
            stdout=subprocess.PIPE,
        )
        if process.stdout:
            logger.info(f"      stdout/stderr:\n{process.stdout}")
        if process.returncode:
            raise Exception("execute_after command failed, see logs above for details.")


def _process_get_file_action(
    base_target_dir: Path, source: str, action_data: dict[str, Any]
):
    """downloads one file to a given location"""
    target_file = action_data.get("target_file", None)
    if not target_file:
        raise Exception("target_file is mandatory when action='get_file'")
    target_dir = action_data.get("target_dir", None)
    local_dir = base_target_dir
    if target_dir:
        local_dir = base_target_dir / str(target_dir)
        local_dir.mkdir(parents=True, exist_ok=True)
    local_file = local_dir / str(target_file)
    if local_file.exists():
        logger.info("    Skipping, local_file is already present")
        return
    _download_file(source, local_file)


def _process_extract_all_action(
    base_target_dir: Path, source: str, action_data: dict[str, Any]
):
    """extacts all Zip content to a given location

    Supports the remove attribute which allows to delete some Zip content
    """
    target_dir = action_data.get("target_dir", None)
    if not target_dir:
        raise Exception("target_dir is mandatory when action='extract_all'")
    target_dir = base_target_dir / str(target_dir)
    if target_dir.exists():
        logger.info("    Skipping, target_dir is already present")
        return
    if not target_dir.parent.exists():
        target_dir.parent.mkdir(parents=True, exist_ok=True)
    _extract_zip_from_url(url=source, extract_to=target_dir)
    if "remove" in action_data:
        _remove_items(globs=action_data["remove"], directory=target_dir)


def _process_extract_items_action(
    base_target_dir: Path, source: str, action_data: dict[str, Any]
):
    """extacts some items of a content to some given locations

    Supports the remove attribute which allows to delete some items content after
    extraction
    """
    zip_paths = action_data.get("zip_paths", None)
    if not zip_paths:
        raise Exception("zip_paths is mandatory when action='extract_items'")
    target_paths = action_data.get("target_paths", None)
    if not target_paths:
        raise Exception("target_paths is mandatory when action='extract_items'")
    if len(zip_paths) != len(target_paths):
        raise Exception(
            f"zip_paths and target_paths must have same length ({len(zip_paths)} !="
            f" {len(target_paths)})"
        )

    # do not re-install if asset has already been installed
    if any(
        (base_target_dir / str(target_path)).exists() for target_path in target_paths
    ):
        logger.info("    Skipping, at least one target path is already present")
        return

    with tempfile.TemporaryDirectory() as tempdir:
        tempath = Path(tempdir)
        _extract_zip_from_url(url=source, extract_to=tempath)
        for index, zip_path in enumerate(zip_paths):
            item_src = tempath / str(zip_path)
            item_dst = base_target_dir / str(target_paths[index])
            if item_dst.parent and not item_dst.parent.exists():
                item_dst.parent.mkdir(parents=True, exist_ok=True)
            shutil.move(src=str(item_src), dst=item_dst)

    if "remove" in action_data:
        _remove_items(globs=action_data["remove"], directory=base_target_dir)


def _remove_items(directory: Path, globs: list[str]):
    """removes all files in directory matching one of the provided glob patterns"""
    for pattern in globs:
        matches = directory.glob(pattern)
        for match in matches:
            if match.is_file():
                match.unlink()
            elif match.is_dir():  # pragma: no branch
                shutil.rmtree(match)


def _download_file(url: str, download_to: Path):
    """downloads a file to a given location"""
    if not url.startswith(("http:", "https:")):
        raise ValueError("URL must start with 'http:' or 'https:'")
    with urlopen(url) as response, open(download_to, "wb") as file:  # noqa: S310
        file.write(response.read())


def _extract_zip_from_url(url: str, extract_to: Path):
    """downloads ZIP from URL and extract in given directory

    Nota: the ZIP is temporarily saved on disk (there is no convenient function
    to stream the web resource to the ZIP extractor in Python standard library)
    """

    if not url.startswith(("http:", "https:")):
        raise ValueError("URL must start with 'http:' or 'https:'")
    with urlopen(url) as response:  # noqa: S310
        with tempfile.NamedTemporaryFile(delete=False) as temp_zip:
            shutil.copyfileobj(response, temp_zip)
        with zipfile.ZipFile(temp_zip.name, "r") as zip_file:
            zip_file.extractall(extract_to)
        Path(temp_zip.name).unlink()



import tempfile
from collections.abc import Callable, Generator
from contextlib import contextmanager
from pathlib import Path
from typing import Any

import pytest

from hatch_openzim.utils import GithubInfo, get_github_info, get_python_versions


@pytest.fixture
def mock_git_config() -> Generator[Callable[[str, str], Any], None, None]:
    @contextmanager
    def _mock_git_config(git_origin_url: str, remote_name: str = "origin"):
        with tempfile.NamedTemporaryFile() as temp_file:
            git_config = Path(temp_file.name)
            git_config.write_text(
                f"""
[core]
        repositoryformatversion = 0
        filemode = true
        bare = false
        logallrefupdates = true
[remote "{remote_name}"]
        url = {git_origin_url}
        fetch = +refs/heads/*:refs/remotes/origin/*
"""
            )
            yield git_config

    yield _mock_git_config


@pytest.mark.parametrize(
    "git_url, expected_homepage_url, expected_organization, expected_repository",
    [
        (
            "https://github.com/oneuser/onerepo.git",
            "https://github.com/oneuser/onerepo",
            "oneuser",
            "onerepo",
        ),
        (
            "https://github.com/oneuser/onerepo",
            "https://github.com/oneuser/onerepo",
            "oneuser",
            "onerepo",
        ),
        (
            "git@github.com:oneuser/one-repo.git",
            "https://github.com/oneuser/one-repo",
            "oneuser",
            "one-repo",
        ),
    ],
)
def test_get_github_project_homepage_valid_url(
    mock_git_config: Callable[[str], Any],
    git_url: str,
    expected_homepage_url: str,
    expected_organization: str,
    expected_repository: str,
):
    with mock_git_config(git_url) as git_config_path:
        assert get_github_info(git_config_path=git_config_path) == GithubInfo(
            homepage=expected_homepage_url,
            organization=expected_organization,
            repository=expected_repository,
        )


def test_get_github_project_homepage_invalid_url(mock_git_config: Callable[[str], Any]):
    # Test the function with an invalid URL
    with mock_git_config("http://github.com/oneuser/onerepo.git") as git_config_path:
        assert get_github_info(git_config_path=git_config_path) == GithubInfo(
            homepage="https://www.kiwix.org", organization=None, repository=None
        )


def test_get_github_project_missing_git_config():
    # Test the function with an invalid URL
    assert get_github_info(git_config_path=Path("i_m_not_here.config")) == GithubInfo(
        homepage="https://www.kiwix.org", organization=None, repository=None
    )


def test_get_github_project_homepage_invalid_remote(
    mock_git_config: Callable[[str, str], Any],
):
    # Test the function with an invalid URL
    with mock_git_config(
        "https://github.com/oneuser/onerepo.git", "origin2"
    ) as git_config_path:
        assert get_github_info(git_config_path=git_config_path) == GithubInfo(
            homepage="https://www.kiwix.org", organization=None, repository=None
        )


@pytest.mark.parametrize(
    "requires_python, expected_versions",
    [
        (
            ">=3.1,<3.2",
            ["3", "3.1"],
        ),
        (
            ">=3.10,<3.12",
            ["3", "3.10", "3.11"],
        ),
        (
            ">=2.4,<3.1",
            ["2", "2.4", "2.5", "2.6", "2.7", "3", "3.0"],
        ),
    ],
)
def test_get_python_versions_ok(requires_python: str, expected_versions: list[str]):
    python_versions = get_python_versions(requires_python)
    # we compare sets because order does not matter
    assert set(python_versions) == set(expected_versions)
    # we compare length because we do not want duplicated values
    assert len(set(python_versions)) == len(python_versions)


@pytest.mark.parametrize(
    "requires_python",
    [
        (">=3.10,<4.1"),
        (">=3.6"),
    ],
)
def test_get_python_versions_ko(requires_python: str):
    with pytest.raises(
        Exception, match="Multiple major versions is not supported for 3 and up"
    ):
        get_python_versions(requires_python)



import logging
import os
import sys

DEFAULT_FORMAT = "%(name)s:%(levelname)s:%(message)s"

# create logger
logger = logging.getLogger("hatch_openzim")
logger.setLevel(logging.DEBUG)

# setup console logging
console_handler = logging.StreamHandler(sys.stdout)
console_handler.setFormatter(logging.Formatter(DEFAULT_FORMAT))
console_handler.setLevel(os.getenv("HATCH_OPENZIM_LOG_LEVEL", "INFO"))
logger.addHandler(console_handler)



from typing import Any

from hatchling.builders.hooks.plugin.interface import BuildHookInterface

from hatch_openzim.files_install import process as process_files_install


class OpenzimBuildHook(
    BuildHookInterface  # pyright: ignore[reportMissingTypeArgument]
):
    """Hatch build hook to perform custom openzim actions

    This hook performs:
    - files installation
    """

    PLUGIN_NAME = "openzim-build"

    def initialize(self, version: str, build_data: dict[str, Any]):  # noqa: ARG002
        if "toml-config" in self.config:
            process_files_install(openzim_toml_location=self.config["toml-config"])
        else:
            process_files_install()



import os
import shutil
from pathlib import Path
from typing import Any

import pytest

from hatch_openzim.metadata import update

Metadata = dict[str, str | list[str]]


@pytest.fixture
def dynamic_metadata() -> list[str]:
    return [
        "authors",
        "classifiers",
        "keywords",
        "license",
        "urls",
    ]


@pytest.fixture
def root_folder(tmp_path: Path) -> str:
    """
    Returns a "virtual" root folder with a "virtual" git config

    Git config comes from the tests/configs/gitconfig file

    This is necessary to ensure tests run always with the same git configuration file,
    to avoid variability coming from:
    - tests ran on plain files (not linked to any git repo)
    - tests ran on a repository fork (e.g myuser/hatch-openzim)
    - tests ran with a different remote (nothing forces main remote to be named origin)
    """
    git_folder = tmp_path / ".git"
    git_folder.mkdir()
    shutil.copy(
        Path(os.path.dirname(os.path.abspath(__file__))).parent
        / "tests/configs/gitconfig",
        git_folder / "config",
    )
    return str(tmp_path)


@pytest.fixture
def metadata(dynamic_metadata: list[str]) -> Metadata:
    return {
        "requires-python": ">=3.10,<3.12",
        "dynamic": dynamic_metadata,
    }


def test_metadata_nominal(metadata: Metadata, root_folder: str):
    update(
        root=root_folder,
        config={},
        metadata=metadata,
    )

    assert metadata["authors"] == [{"email": "dev@openzim.org", "name": "openZIM"}]
    assert metadata["classifiers"] == [
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ]
    assert metadata["keywords"] == ["openzim"]
    assert metadata["license"] == {"text": "GPL-3.0-or-later"}
    assert metadata["urls"] == {
        "Donate": "https://www.kiwix.org/en/support-us/",
        "Homepage": "https://github.com/openzim/hatch-openzim",
    }


@pytest.mark.parametrize(
    "metadata_key",
    [
        ("authors"),
        ("classifiers"),
        ("keywords"),
        ("license"),
        ("urls"),
    ],
)
def test_metadata_missing_dynamic(
    metadata: Metadata, metadata_key: str, root_folder: str
):
    assert isinstance(metadata["dynamic"], list)
    assert all(isinstance(item, str) for item in metadata["dynamic"])
    metadata["dynamic"].remove(metadata_key)
    with pytest.raises(
        Exception,
        match=f"'{metadata_key}' must be listed in 'project.dynamic' when using openzim"
        " metadata hook",
    ):
        update(
            root=root_folder,
            config={},
            metadata=metadata,
        )


@pytest.mark.parametrize(
    "metadata_key",
    [
        ("authors"),
        ("classifiers"),
        ("keywords"),
        ("license"),
        ("urls"),
    ],
)
def test_metadata_metadata_already_there(
    metadata: Metadata, metadata_key: str, root_folder: str
):
    metadata[metadata_key] = "some_value"
    with pytest.raises(
        Exception,
        match=f"'{metadata_key}' must not be listed in the 'project' table when using "
        "openzim metadata hook",
    ):
        update(
            root=root_folder,
            config={},
            metadata=metadata,
        )


@pytest.mark.parametrize(
    "metadata_key",
    [
        ("authors"),
        ("classifiers"),
        ("keywords"),
        ("license"),
        ("urls"),
    ],
)
def test_metadata_preserve_value(
    metadata: Metadata, metadata_key: str, root_folder: str
):
    metadata[metadata_key] = f"some_value_for_{metadata_key}"
    config: dict[str, Any] = {}
    config[f"preserve-{metadata_key}"] = True
    update(
        root=root_folder,
        config=config,
        metadata=metadata,
    )
    assert metadata[metadata_key] == f"some_value_for_{metadata_key}"


def test_metadata_additional_keywords(metadata: Metadata, root_folder: str):
    config: dict[str, Any] = {}
    config["additional-keywords"] = ["keyword1", "keyword2"]
    update(
        root=root_folder,
        config=config,
        metadata=metadata,
    )
    # we compare sets because order is not relevant
    assert set(metadata["keywords"]) == {"openzim", "keyword1", "keyword2"}


def test_metadata_additional_classifiers(metadata: Metadata, root_folder: str):
    config: dict[str, Any] = {}
    config["additional-classifiers"] = [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
    ]
    update(
        root=root_folder,
        config=config,
        metadata=metadata,
    )
    # we compare sets because order is not relevant
    assert set(metadata["classifiers"]) == {
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
    }


def test_metadata_additional_authors(metadata: Metadata, root_folder: str):
    config: dict[str, Any] = {}
    config["additional-authors"] = [{"email": "someone@acme.org", "name": "Some One"}]
    update(
        root=root_folder,
        config=config,
        metadata=metadata,
    )
    # we compare sets because order is not relevant
    assert metadata["authors"] == [
        {"email": "dev@openzim.org", "name": "openZIM"},
        {"email": "someone@acme.org", "name": "Some One"},
    ]


@pytest.mark.parametrize(
    "organization, expected_result",
    [
        ("kiwix", "kiwix"),
        ("Kiwix", "kiwix"),
        ("openzim", "openzim"),
        ("openZIM", "openzim"),
        ("offspot", "kiwix"),
        ("unknown", "openzim"),
        (None, "openzim"),
    ],
)
def test_metadata_organization(
    organization: str, expected_result: str, metadata: Metadata, root_folder: str
):
    config: dict[str, Any] = {}
    if organization:
        config["organization"] = organization
    update(
        root=root_folder,
        config=config,
        metadata=metadata,
    )
    if expected_result == "kiwix":
        assert metadata["authors"] == [{"email": "dev@kiwix.org", "name": "Kiwix"}]
        assert metadata["keywords"] == ["kiwix"]
    elif expected_result == "openzim":
        assert metadata["authors"] == [{"email": "dev@openzim.org", "name": "openZIM"}]
        assert metadata["keywords"] == ["openzim"]
    else:
        raise Exception(f"Unexpected expected result: {expected_result}")


def test_metadata_is_scraper(metadata: Metadata, root_folder: str):
    config: dict[str, Any] = {}
    config["kind"] = "scraper"
    update(
        root=root_folder,
        config=config,
        metadata=metadata,
    )
    # we compare sets because order is not relevant
    assert set(metadata["keywords"]) == {"openzim", "offline", "zim"}



import configparser
import re
from pathlib import Path
from typing import NamedTuple

from packaging.specifiers import SpecifierSet
from packaging.version import Version

from hatch_openzim.shared import logger

REMOTE_REGEXP = re.compile(
    r"""^(?:git@|https:\/\/)github.com[:\/](?P<organization>.*?)\/"""
    r"""(?P<repository>.*?)(?:.git)?$"""
)


class GithubInfo(NamedTuple):
    homepage: str
    organization: str | None
    repository: str | None


DEFAULT_GITHUB_INFO = GithubInfo(
    homepage="https://www.kiwix.org", organization=None, repository=None
)


def get_github_info(git_config_path: Path, remote: str = "origin") -> GithubInfo:
    if not git_config_path.exists() or not git_config_path.is_file():
        return DEFAULT_GITHUB_INFO

    try:
        config = configparser.ConfigParser()
        config.read(git_config_path)
        git_remote_url = config[f'remote "{remote}"']["url"]
        match = REMOTE_REGEXP.match(git_remote_url)
        if not match:
            raise Exception(f"Unexpected remote url: {git_remote_url}")
        return GithubInfo(
            homepage=f"https://github.com/{match.group('organization')}/"
            f"{match.group('repository')}",
            organization=match.group("organization"),
            repository=match.group("repository"),
        )
    except Exception as exc:
        logger.error("Failed to read Github URL", exc_info=exc)
        return DEFAULT_GITHUB_INFO


def get_python_versions(requires_python: str) -> list[str]:
    """
    Returns the list of major and major.minor versions compatible with the specifier

    E.g. if requires_python is ">=3.10,<3.12", the result is "3", "3.10", "3.11"

    Nota: this does not work for requirements overlapping 3.x and 4.x, or later, because
     latest 3.x version is not yet known
    """
    specifier_set = SpecifierSet(requires_python)

    last_py1_minor = 6
    last_py2_minor = 7

    major_versions: list[str] = []
    minor_versions: list[str] = []
    for major in range(1, 10):  # this will work up to Python 10 ...
        major_added = False
        last_minor = 100  # this supposes we will never have Python x.100
        if major == 1:
            last_minor = last_py1_minor
        elif major == 2:  # noqa: PLR2004
            last_minor = last_py2_minor
        for minor in range(last_minor + 1):
            if specifier_set.contains(Version(f"{major}.{minor}")):
                if not major_added:
                    if len(major_versions) > 0 and major >= 4:  # noqa: PLR2004
                        raise Exception(
                            "Multiple major versions is not supported for 3 and up"
                        )
                    major_added = True
                    major_versions.append(f"{major}")
                minor_versions.append(f"{major}.{minor}")

    return major_versions + minor_versions



from hatchling.plugin import hookimpl

from hatch_openzim.build_hook import OpenzimBuildHook
from hatch_openzim.metadata_hook import OpenzimMetadataHook


@hookimpl
def hatch_register_build_hook() -> type[OpenzimBuildHook]:
    return OpenzimBuildHook


@hookimpl
def hatch_register_metadata_hook() -> type[OpenzimMetadataHook]:
    return OpenzimMetadataHook



from pathlib import Path
from typing import Any

from hatch_openzim.utils import get_github_info, get_python_versions


def update(root: str, config: dict[str, Any], metadata: dict[str, Any]):
    """Update the project table's metadata."""

    # Check for absence of metadata we will set + presence in the dynamic property
    for metadata_key in ["urls", "authors", "keywords", "license", "classifiers"]:
        if config.get(f"preserve-{metadata_key}", False):
            # Do not check if we intend to preserve the value set manually
            continue

        if metadata_key in metadata:
            raise ValueError(
                f"'{metadata_key}' must not be listed in the 'project' table when using"
                " openzim metadata hook."
            )
        if metadata_key not in metadata.get("dynamic", []):
            raise ValueError(
                f"'{metadata_key}' must be listed in 'project.dynamic' when using "
                "openzim metadata hook."
            )

    github_info = get_github_info(git_config_path=Path(root) / ".git/config")

    organization = config.get("organization", github_info.organization)

    if not config.get("preserve-urls", False):
        metadata["urls"] = {
            "Donate": "https://www.kiwix.org/en/support-us/",
            "Homepage": github_info.homepage,
        }

    if not config.get("preserve-authors", False):
        if str(organization).lower() in ("kiwix", "offspot"):
            authors = [{"name": "Kiwix", "email": "dev@kiwix.org"}]
        else:
            authors = [{"name": "openZIM", "email": "dev@openzim.org"}]
        authors.extend(config.get("additional-authors", []))
        metadata["authors"] = authors

    if not config.get("preserve-keywords", False):
        if str(organization).lower() in ("kiwix", "offspot"):
            keywords = ["kiwix"]
        else:
            keywords = ["openzim"]
        if config.get("kind", "") == "scraper":
            keywords.extend(["zim", "offline"])
        keywords.extend(config.get("additional-keywords", []))
        metadata["keywords"] = keywords

    if not config.get("preserve-license", False):
        metadata["license"] = {"text": "GPL-3.0-or-later"}

    if not config.get("preserve-classifiers", False):
        classifiers = [
            "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)"
        ]
        for python_version in get_python_versions(metadata["requires-python"]):
            classifiers.append(f"Programming Language :: Python :: {python_version}")
        classifiers.extend(config.get("additional-classifiers", []))
        metadata["classifiers"] = classifiers



import os
import tempfile
from pathlib import Path

import pytest

from hatch_openzim import files_install


@pytest.fixture
def nominal_files():
    return [
        "part1/action1/file1.txt",
        "part1/action1/file2.txt",
        "part1/action1/keep1/file1.txt",
        "part1/action1/keep1/file2.txt",
        "part1/action1/remove3/file2.txt",
        "part1/somewhere/something.txt",
        "part1/somewhere_else/something.txt",
        "part2/file123.txt",
        "part2/action5/subfolder1/file123.txt",
        "part2/action2/file1.txt",
        "part2/action2/file2.txt",
        "part2/action3/file1.json",
        "part2/action3/file1.txt",
        "part2/action3/file2.json",
        "part2/action3/file2.txt",
        "part4/action2/file4.txt",
        "part4/file4.txt",
        "part4/subdir1/action3/file1.txt",
        "part4/subdir1/action3/file2.txt",
        "part4/subdir1/action3/keep1/file1.txt",
        "part4/subdir1/action3/keep1/file2.txt",
        "part4/subdir1/action3/remove1/file1.txt",
        "part4/subdir1/action3/remove1/file2.txt",
        "part4/subdir1/action3/remove2.txt",
        "part4/subdir1/action3/remove3/file1.txt",
        "part4/subdir1/action3/remove3/file2.txt",
    ]


def test_no_arg():
    """Test default case where no config is passed and file is missing"""
    files_install.process()


@pytest.mark.parametrize(
    "config_file",
    [
        ("empty.toml"),
        ("other_stuff.toml"),
    ],
)
def test_ignored_silently(config_file: str):
    """Test cases where the config file is passed but there is no relevant content"""
    files_install.process(
        str((Path(__file__).parent / "configs" / config_file).absolute())
    )


@pytest.mark.parametrize(
    "config_file, exception_message",
    [
        ("i_m_not_here.toml", "File is missing at .*/i_m_not_here.toml"),
        ("missing_config.toml", "config table is mandatory"),
        ("missing_target_dir.toml", "target_dir is mandatory in config table"),
        ("missing_source.toml", "source is not configured"),
        ("missing_action.toml", "action is not configured"),
        (
            "get_file_missing_target_file.toml",
            "target_file is mandatory when action='get_file'",
        ),
        (
            "extract_all_missing_target_dir.toml",
            "target_dir is mandatory when action='extract_all'",
        ),
        ("unsupported_protocol_dl.toml", "URL must start with 'http:' or 'https:'"),
        ("unsupported_protocol_zip.toml", "URL must start with 'http:' or 'https:'"),
        (
            "extract_items_missing_zip_paths.toml",
            "zip_paths is mandatory when action='extract_items'",
        ),
        (
            "extract_items_missing_target_paths.toml",
            "target_paths is mandatory when action='extract_items'",
        ),
        (
            "extract_items_length_not_matching.toml",
            "zip_paths and target_paths must have same length .*",
        ),
        ("unsupported_action.toml", "Unsupported action 'unkwown'"),
    ],
)
def test_errors(config_file: str, exception_message: str):
    """Tests cases with improper configurations"""
    with pytest.raises(Exception, match=exception_message):
        files_install.process(
            str((Path(__file__).parent / "configs" / config_file).absolute())
        )


def test_full(nominal_files: list[str]):
    """Nominal test cases where many files are installed"""

    # Proceed with installation in a temporary directory
    with tempfile.TemporaryDirectory() as temp_dir:
        os.chdir(temp_dir)

        # Run the file installation
        files_install.process(
            str((Path(__file__).parent / "configs" / "full.toml").absolute())
        )

        # Compare final directory content
        existing_files = [
            str(file.relative_to(temp_dir)) for file in Path(temp_dir).rglob("*.*")
        ]
        # compare set and len, since order does not matter
        assert set(existing_files) == set(nominal_files)
        assert len(existing_files) == len(nominal_files)


def test_assets_already_there_extract_all(nominal_files: list[str]):
    """Assets have already been installed for extract_all action"""

    # Proceed with installation in a temporary directory
    with tempfile.TemporaryDirectory() as temp_dir:
        os.chdir(temp_dir)

        # Create a file which is supposed to be replaced by extract_all
        Path(temp_dir, "part1/action1").mkdir(parents=True, exist_ok=True)
        Path(temp_dir, "part1/action1/something.txt").touch()

        # Run the file installation
        files_install.process(
            str((Path(__file__).parent / "configs" / "full.toml").absolute())
        )

        # Compare final directory content
        existing_files = [
            str(file.relative_to(temp_dir)) for file in Path(temp_dir).rglob("*.*")
        ]
        expected_files = [
            file for file in nominal_files if not file.startswith("part1/action1")
        ] + ["part1/action1/something.txt"]
        # compare set and len, since order does not matter
        assert set(existing_files) == set(expected_files)
        assert len(existing_files) == len(expected_files)


def test_assets_already_there_extract_items(nominal_files: list[str]):
    """Assets have already been installed for extract_items action"""

    # Proceed with installation in a temporary directory
    with tempfile.TemporaryDirectory() as temp_dir:
        os.chdir(temp_dir)

        # Create a file which is supposed to be replaced by extract_items
        Path(temp_dir, "part2/action2").mkdir(parents=True, exist_ok=True)
        Path(temp_dir, "part2/action2/something.txt").touch()

        # Run the file installation
        files_install.process(
            str((Path(__file__).parent / "configs" / "full.toml").absolute())
        )

        # Compare final directory content
        existing_files = [
            str(file.relative_to(temp_dir)) for file in Path(temp_dir).rglob("*.*")
        ]
        expected_files = [
            file for file in nominal_files if not file.startswith("part2/action2")
        ] + ["part2/action2/something.txt"]
        # compare set and len, since order does not matter
        assert set(existing_files) == set(expected_files)
        assert len(existing_files) == len(expected_files)


def test_assets_already_there_get_file(nominal_files: list[str]):
    """Assets have already been installed for get_file action"""

    # Proceed with installation in a temporary directory
    with tempfile.TemporaryDirectory() as temp_dir:
        os.chdir(temp_dir)

        # Create a file which is supposed to be replaced by get_file
        Path(temp_dir, "part4").mkdir(parents=True, exist_ok=True)
        Path(temp_dir, "part4/file4.txt").touch()

        # Run the file installation
        files_install.process(
            str((Path(__file__).parent / "configs" / "full.toml").absolute())
        )

        # Compare final directory content
        existing_files = [
            str(file.relative_to(temp_dir)) for file in Path(temp_dir).rglob("*.*")
        ]
        # compare set and len, since order does not matter
        assert set(existing_files) == set(nominal_files)
        assert len(existing_files) == len(nominal_files)
        assert Path(temp_dir, "part4/file4.txt").lstat().st_size == 0


def test_execute_after_failure():
    """Test case where the execute after command is failing"""

    # Proceed with installation in a temporary directory
    with tempfile.TemporaryDirectory() as temp_dir:
        os.chdir(temp_dir)

        with pytest.raises(
            Exception,
            match=r"execute\_after command failed, see logs above for details.",
        ):
            files_install.process(
                str(
                    (
                        Path(__file__).parent / "configs/execute_after_failure.toml"
                    ).absolute()
                )
            )



from typing import Any

from hatchling.metadata.plugin.interface import MetadataHookInterface

from hatch_openzim.metadata import update


class OpenzimMetadataHook(MetadataHookInterface):
    """Hatch metadata hook to populate 'project' data

    This hook populates:
    - project urls
    """

    PLUGIN_NAME = "openzim-metadata"

    def update(self, metadata: dict[str, Any]):
        """Update the project table's metadata."""
        update(
            root=self.root,
            config=self.config,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
            metadata=metadata,
        )
